@@ -18,7 +18,7 @@ from aizoo.utils.model_utils import get_imp
 
 class AdversarialValidation(object):
     """Adversarial Validation
-    通过对抗验证确定最优折数
+    通过对抗验证确定最优拆分折数
     """
 
     def __init__(self, params=None, importance_type='split', fit_params=None):
@@ -74,12 +74,14 @@ class AdversarialValidation(object):
         _ = np.array(metrics)
         print(f"\n\033[94mValid: {_.mean():.6f} +/- {_.std():.6f} \033[0m\n")
 
-        return _.mean()
+        self.metrics = _  # auc、std 越小说明拆分的数据分布越接近
+
+        return _.mean(), _.std()
 
 
 class OOF(object):
 
-    def __init__(self, params=None, fit_params=None, weight_func=None, task='Classifier', importance_type='split'):
+    def __init__(self, params=None, fit_params=None, task='Classifier', importance_type='split'):
         """
 
         @param params:
@@ -87,41 +89,47 @@ class OOF(object):
         @param weight_func:
         @param task: Classifier or Regressor
         """
+        self.task = task.title()
         self.params = params if params is not None else {}
         self.fit_params = fit_params if fit_params is not None else {}
-        self.weight_func = weight_func
-
-        self.task = task.title()
 
         self._estimators = []  # 每一折的模型
         self._importance_type = importance_type
         self.feature_importances_ = None
 
     @abstractmethod
-    def fit_predict(self, X_train, y_train, X_valid, y_valid, X_test, **kwargs):
+    def fit_predict(self, X_train, y_train, w_train, X_valid, y_valid, w_valid, X_test, **kwargs):
         raise NotImplementedError
 
+    @staticmethod
+    def _predict(estimator):
+        return estimator.predict_proba if hasattr(estimator, 'predict_proba') else estimator.predict
+
     def predict(self, X):
-        preds = []
 
-        for estimator in self._estimators:
-            if hasattr(estimator, 'predict_proba'):
-                preds.append(estimator.predict_proba(X))
-            else:
-                preds.append(estimator.predict(X))
+        # preds = []
+        # for estimator in self._estimators:
+        #     _ = self._predict(estimator)(X)
+        #     preds.append(_)
+        #
+        # return np.array(preds).mean(0)
 
-        return np.array(preds).mean(0)
+        func = lambda e: self._predict(e)(X)
+        return np.array(list(map(func, self._estimators))).mean(0)
 
-    def fit(self, X, y, X_test=None, feval=None, cv=5, split_seed=777, oof_file=None):
+    def fit(self, X, y, sample_weight=None, X_test=None, feval=None, cv=5, split_seed=777, oof_file=None):
+
+        if sample_weight is None:
+            sample_weight = np.ones(len(X))
 
         X_test = X_test if X_test is not None else X[:66]
 
         self.feature_importances_ = np.zeros((cv, X.shape[1]))
 
-        if self.task == 'Regressor':  # 目标值多于128个就认为是回归问题 len(set(y)) > 128
+        if self.task == 'Regressor':
             self.oof_train_proba = np.zeros(len(X))
             self.oof_test_proba = np.zeros(len(X_test))
-            _ = enumerate(ShuffleSplit(cv, random_state=split_seed).split(X, y))
+            _ = enumerate(ShuffleSplit(cv, random_state=split_seed).split(X, y))  # todo: 兼容时间序列
 
         elif self.task == 'Classifier':
             num_classes = len(set(y))
@@ -133,13 +141,13 @@ class OOF(object):
             raise ValueError("TaskTypeError⚠️")
 
         valid_metrics = []
-        for n_fold, (train_index, valid_index) in tqdm(_, desc='Train🔥'):
+        for n_fold, (train_index, valid_index) in tqdm(_, desc='Train 🐢'):
             print(f"\033[94mFold {n_fold + 1} started at {time.ctime()}\033[0m")
-            X_train, y_train = X[train_index], y[train_index]
-            X_valid, y_valid = X[valid_index], y[valid_index]
+            X_train, y_train, w_train = X[train_index], y[train_index], sample_weight[train_index]
+            X_valid, y_valid, w_valid = X[valid_index], y[valid_index], sample_weight[valid_index]
 
             ##############################################################
-            valid_predict, test_predict = self.fit_predict(X_train, y_train, X_valid, y_valid, X_test)
+            valid_predict, test_predict = self.fit_predict(X_train, y_train, w_train, X_valid, y_valid, w_valid, X_test)
             ##############################################################
 
             self.oof_train_proba[valid_index] = valid_predict
@@ -176,10 +184,10 @@ class OOF(object):
 
     @classmethod
     def opt_cv(cls, X, y, X_test=None, cv_list=range(3, 16), params=None, **run_kwargs):
-        """todo: 折数优化"""
+        """todo: 折数优化，看方差？"""
 
         scores = []
-        for cv in tqdm(cv_list, desc='opt cv🔥'):  # range(3, 16):
+        for cv in tqdm(cv_list, desc='opt cv 🐢'):  # range(3, 16):
             oof = cls(params)
             oof.fit(X, y, X_test, **run_kwargs)
             scores.append((oof.oof_score, cv, oof))
@@ -203,5 +211,6 @@ class OOF(object):
         sns.barplot(x=columns[0], y=columns[1], data=self.df_feature_importances[:topk])
         plt.title(f'Features {self._importance_type.title()} Importances')
         plt.tight_layout()
+
         if pic_name is not None:
             plt.savefig(f'importances_{self.oof_score}.png')
